@@ -5,7 +5,7 @@
 #  open source, and has the attribution requirements (GPL Section 7) at
 #  https://statnet.org/attribution .
 #
-#  Copyright 2015-2024 Statnet Commons
+#  Copyright 2015-2025 Statnet Commons
 ################################################################################
 data(egor32, package="egor")
 
@@ -20,6 +20,10 @@ for( v in varnames ) {
     })
 }
 
+test_that("Vertex attribute not found", {
+  expect_error(mixingmatrix(egor32, "abc"),
+               "vertex attribute 'abc' not found in egocentric dataset 'egor32'")
+})
 
 for( v in varnames ) {
   test_that(
@@ -94,3 +98,21 @@ test_that("mixing matrices for FMH and egoFMH are equivalent", {
     }
   )
 })
+
+test_that("mixing matrices for FMH and egoFMH are equivalent with missing data", {
+  data("faux.mesa.high")
+  set.vertex.attribute(faux.mesa.high, "Grade", NA, 1:40)
+  fmh.ego <- as.egor(faux.mesa.high)
+  expect_equal(
+    {
+      mm.ego <- mixingmatrix(fmh.ego, "Grade")
+      names(dimnames(mm.ego)) <- c("From", "To")
+      mm.ego
+    },
+    {
+      mm <- mixingmatrix(faux.mesa.high, "Grade")
+      diag(mm) <- diag(mm) * 2
+      mm
+    }
+  )
+})
